# -*- coding: utf-8 -*-
"""
Enterprise Data Django application initialization.
"""

from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class EnterpriseDataRolesConfig(AppConfig):
    """
    Configuration for the Enterprise Data Roles Django application.
    """

    name = 'enterprise_data_roles'
